@@ -1,11 +1,11 @@
 /*
  * radius.c	Functions to send/receive radius packets.
  *
- * Version:	$Id: radius.c,v 1.77 2002/06/24 09:41:14 fcusack Exp $
+ * Version:	$Id: radius.c,v 1.79 2002/07/26 14:43:21 aland Exp $
  *
  */
 
-static const char rcsid[] = "$Id: radius.c,v 1.77 2002/06/24 09:41:14 fcusack Exp $";
+static const char rcsid[] = "$Id: radius.c,v 1.79 2002/07/26 14:43:21 aland Exp $";
 
 #include	"autoconf.h"
 #include	"md5.h"
@@ -385,6 +385,10 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 					         lvalue = htonl(reply->lvalue);
 					  }
 				  } else {
+					  /*
+					   *  IP address is already in
+					   *  network byte order.
+					   */
 					  lvalue = reply->lvalue;
 				  }
 				  memcpy(ptr, &lvalue, 4);
@@ -392,6 +396,18 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 				  total_length += 4;
 				  break;
 
+				  /*
+				   *  There are no tagged date attributes.
+				   */
+			  case PW_TYPE_DATE:
+				  *length_ptr += 4;
+				  if (vsa_length_ptr) *vsa_length_ptr += 4;
+
+				  lvalue = htonl(reply->lvalue);
+				  memcpy(ptr, &lvalue, 4);
+				  ptr += 4;
+				  total_length += 4;
+				  break;
 			  default:
 				  break;
 			  }
@@ -1202,16 +1218,18 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 			if (attrlen != 4) {
 				pair->type = PW_TYPE_OCTETS;
 				memcpy(pair->strvalue, ptr, attrlen);
-				pair->lvalue = 0xbaddbadd;
+				pair->lvalue = 0xbad1bad1;
 				break;
 			}
 
       			memcpy(&lvalue, ptr, 4);
 
-			if (attr->type != PW_TYPE_IPADDR)
+			if (attr->type != PW_TYPE_IPADDR) {
 				pair->lvalue = ntohl(lvalue);
-			else
+				ip_ntoa(pair->strvalue, pair->lvalue);
+			} else {
 				pair->lvalue = lvalue;
+			}
 
 			/*
 			 *  Only PW_TYPE_INTEGER should have tags.
