@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: conffile.c,v 1.84 2002/05/29 16:18:35 aland Exp $
+ * Version:	$Id: conffile.c,v 1.85 2002/07/04 15:02:10 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 #include "modules.h"
 
 static const char rcsid[] =
-"$Id: conffile.c,v 1.84 2002/05/29 16:18:35 aland Exp $";
+"$Id: conffile.c,v 1.85 2002/07/04 15:02:10 aland Exp $";
 
 #define xstrdup strdup
 
@@ -786,7 +786,7 @@ static int xlat_config(void *instance, REQUEST *request,
 	
 	strNcpy(out, cp->value, outlen);
     
-	return outlen;
+	return outlen - 1;
 }
 
 /*
