@@ -2,7 +2,7 @@
  * nas.c	Functions to do with a NASLIST. This is here because
  *		radzap needs it as well.
  *
- * Version:     $Id: nas.c,v 1.17 2002/03/02 05:50:43 ramoore Exp $
+ * Version:     $Id: nas.c,v 1.18 2002/06/17 15:34:32 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: nas.c,v 1.17 2002/03/02 05:50:43 ramoore Exp $";
+static const char rcsid[] = "$Id: nas.c,v 1.18 2002/06/17 15:34:32 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -78,7 +78,7 @@ int read_naslist_file(char *file)
 	}
 	while(fgets(buffer, 256, fp) != NULL) {
 		lineno++;
-		if (strchr(buffer, '\n') == NULL) {
+		if (!feof(fp) && (strchr(buffer, '\n') == NULL)) {
 			radlog(L_ERR, "%s[%d]: line too long", file, lineno);
 			return -1;
 		}
