@@ -1,7 +1,7 @@
 /*
  * proxy.c	Proxy stuff.
  *
- * Version:	$Id: proxy.c,v 1.54 2002/03/07 18:11:20 aland Exp $
+ * Version:	$Id: proxy.c,v 1.56 2002/06/07 13:20:03 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Chris Parker <cparker@starnetusa.com>
  */
 
-static const char rcsid[] = "$Id: proxy.c,v 1.54 2002/03/07 18:11:20 aland Exp $";
+static const char rcsid[] = "$Id: proxy.c,v 1.56 2002/06/07 13:20:03 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -57,8 +57,6 @@ int proxy_receive(REQUEST *request)
 	VALUE_PAIR *replicatepair;
 	VALUE_PAIR *realmpair;
 	int replicating;
-	REALM *realm;
-	char *realmname;
 
 	proxypair = pairfind(request->config_items, PW_PROXY_TO_REALM);
 	replicatepair = pairfind(request->config_items, PW_REPLICATE_TO_REALM);
@@ -73,9 +71,6 @@ int proxy_receive(REQUEST *request)
 		return -1;
 	}
 
-	realmname = (char *) realmpair->strvalue;
-	realm = realm_find(realmname);
-
 	/*
 	 *	Don't touch the reply VP's.  Assume that a module
 	 *	takes care of that...
@@ -175,7 +170,7 @@ int proxy_send(REQUEST *request)
 	 *	If there is no such realm, then exit.
 	 *	Maybe we should log an error?
 	 */
-	realm = realm_find(realmname);
+	realm = realm_find(realmname, (request->packet->code == PW_ACCOUNTING_REQUEST));
 	if (realm == NULL) {
 		return -1;
 	}
