@@ -1,7 +1,7 @@
 /*
  * radzap.c	Zap a user from the radutmp and radwtmp file.
  *
- * Version:	$Id: radzap.c,v 1.28 2002/02/27 15:32:07 aland Exp $
+ * Version:	$Id: radzap.c,v 1.29 2002/07/12 19:45:34 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 #include <string.h>
 #include <netdb.h>
 #include <limits.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 
