@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: threads.c,v 1.44 2002/05/17 18:04:23 aland Exp $
+ * Version:	$Id: threads.c,v 1.45 2002/07/08 18:34:58 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@
 #include "conffile.h"
 
 static const char rcsid[] =
-"$Id: threads.c,v 1.44 2002/05/17 18:04:23 aland Exp $";
+"$Id: threads.c,v 1.45 2002/07/08 18:34:58 aland Exp $";
 
 #define SEMAPHORE_LOCKED	(0)
 #define SEMAPHORE_UNLOCKED	(1)
@@ -502,21 +502,6 @@ int thread_pool_init(void)
 		}
 	}
 
-	/*
-	 *	Initialize the mutex used to remember calls to fork.
-	 */
-	pthread_mutex_init(&fork_mutex, NULL);
-
-	/*
-	 *	Initialize the data structure where we remember the
-	 *	mappings of thread ID && child PID to exit status.
-	 */
-	for (i = 0; i < NUM_FORKERS; i++) {
-		forkers[i].thread_id = NO_SUCH_CHILD_PID;
-		forkers[i].child_pid = -1;
-		forkers[i].status = 0;
-	}
-
 	pool_initialized = TRUE;
 	return 0;
 }
@@ -736,6 +721,29 @@ int thread_pool_clean(time_t now)
 	return 0;
 }
 
+/*
+ *	Initialize the stuff for keeping track of child processes.
+ */
+void rad_exec_init(void)
+{
+	int i;
+	
+	/*
+	 *	Initialize the mutex used to remember calls to fork.
+	 */
+	pthread_mutex_init(&fork_mutex, NULL);
+	
+	/*
+	 *	Initialize the data structure where we remember the
+	 *	mappings of thread ID && child PID to exit status.
+	 */
+	for (i = 0; i < NUM_FORKERS; i++) {
+		forkers[i].thread_id = NO_SUCH_CHILD_PID;
+		forkers[i].child_pid = -1;
+		forkers[i].status = 0;
+	}
+}
+
 /*
  *	Thread wrapper for fork().
  */
@@ -844,13 +852,6 @@ pid_t rad_waitpid(pid_t pid, int *status, int options)
 	int found;
 	pthread_t self = pthread_self();
 
-	/*
-	 *	No threads, nothing magic to do.
-	 */
-	if (!pool_initialized) {
-		return waitpid(pid, status, options);
-	}
-
 	/*
 	 *	We're only allowed to wait for a SPECIFIC pid.
 	 */
@@ -921,13 +922,6 @@ int rad_savepid(pid_t pid, int status)
 {
 	int i;
 
-	/*
-	 *	No threads, nothing magic to do.
-	 */
-	if (!pool_initialized) {
-		return 0;
-	}
-
 	/*
 	 *	Do NOT lock the array, as nothing else sets the
 	 *	status and posts the semaphore.
