@@ -2,7 +2,7 @@
  * valuepair.c	Valuepair functions that are radiusd-specific
  *		and as such do not belong in the library.
  *
- * Version:	$Id: valuepair.c,v 1.39 2002/04/23 15:06:30 cparker Exp $
+ * Version:	$Id: valuepair.c,v 1.41 2002/07/10 19:51:37 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: valuepair.c,v 1.39 2002/04/23 15:06:30 cparker Exp $";
+static const char rcsid[] = "$Id: valuepair.c,v 1.41 2002/07/10 19:51:37 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -272,6 +272,25 @@ int paircmp(REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check, VALUE_PAIR **r
 			return -1;
 		}
 
+		/*
+		 *	We've got to xlat the string before doing
+		 *	the comparison.
+		 */
+		if (check_item->flags.do_xlat) {
+			int rcode;
+			char buffer[sizeof(check_item->strvalue)];
+
+			check_item->flags.do_xlat = 0;
+			rcode = radius_xlat(buffer, sizeof(buffer),
+					    check_item->strvalue,
+					    req, NULL);
+			
+			/*
+			 *	Parse the string into a new value.
+			 */
+			pairparsevalue(check_item, buffer);
+		}
+
 		/*
 		 *	OK it is present now compare them.
 		 */
