@@ -1,7 +1,7 @@
 /*
  * rlm_dbm_parser.c :    Create dbm file from plain text
  *
- * Version:     $Id: rlm_dbm_parser.c,v 1.4 2002/01/14 22:14:27 aland Exp $
+ * Version:     $Id: rlm_dbm_parser.c,v 1.5 2002/06/21 19:57:27 cparker Exp $
  *
  *  This program is is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2 if the
@@ -17,7 +17,7 @@
 
 
 char sccsid[] =
-"$Id: rlm_dbm_parser.c,v 1.4 2002/01/14 22:14:27 aland Exp $ sandy module project\n Copyright 2001 Sandy Service\nCopyright 2001 Koulik Andrei";
+"$Id: rlm_dbm_parser.c,v 1.5 2002/06/21 19:57:27 cparker Exp $ sandy module project\n Copyright 2001 Sandy Service\nCopyright 2001 Koulik Andrei";
 
 #include "autoconf.h"
 #include <fcntl.h>
@@ -207,7 +207,7 @@ static int storecontent (const char * username) {
 static int getuname(char **p,char *u,int n) {
 	int	i; 
 	
-	for(i=0 ; ( i < n-1 ) && ( **p ) && (! isspace(**p) ) ; (*p)++ ) 
+	for(i=0 ; ( i < n-1 ) && ( **p ) && (! isspace((int) **p) ) ; (*p)++ ) 
 	    u[i++] = **p;
 	u[i] = '\0';
 	return ( i == 0) ? 1:0;
@@ -235,7 +235,7 @@ static int sm_parse_file(FILE*fp,const char* fname) {
 		}
 	
 		DOUT2("Parseline: %s",buff);
-		for ( p = buff; isspace(*p); p++);
+		for ( p = buff; isspace((int) *p); p++);
 		 
 		if ( *p == '#' || *p == 0 ) continue;
 
@@ -245,7 +245,7 @@ static int sm_parse_file(FILE*fp,const char* fname) {
 
 		/* try to decide is this line new user or new pattern */
 		if ( parse_state == SMP_PATTERN_OR_USER ) {
-		     if ( isspace(buff[0]) ) parse_state = SMP_PATTERN;
+		     if ( isspace((int) buff[0]) ) parse_state = SMP_PATTERN;
 		     	else {
 		     		parse_state = SMP_USER;
 		     		storecontent(username);
@@ -271,7 +271,7 @@ static int sm_parse_file(FILE*fp,const char* fname) {
 		if ( parse_state == SMP_PATTERN || parse_state == SMP_ACTION ) {
 
 		    /* check for empty line */
-		    while( *p && isspace(*p) ) p++;
+		    while( *p && isspace((int) *p) ) p++;
 
 		    if ( *p && ( *p != ';' ) ) tok = userparse(p,&vp);
 		    else tok = T_EOL;  /* ';' - signs empty line */
@@ -361,7 +361,7 @@ int main(int n,char **argv) {
 				print_stat = 0;
 				break;
 			case 'v':
-				printf("%s: $Id: rlm_dbm_parser.c,v 1.4 2002/01/14 22:14:27 aland Exp $ \n",progname);
+				printf("%s: $Id: rlm_dbm_parser.c,v 1.5 2002/06/21 19:57:27 cparker Exp $ \n",progname);
 				exit(0);
 			case 'c':
 				oflags = O_CREAT | O_TRUNC | O_RDWR;
