@@ -1,7 +1,7 @@
 /*
  * eap.c    rfc2284 & rfc2869 implementation
  *
- * Version:     $Id: eap.c,v 1.4 2002/01/22 21:45:08 raghu Exp $
+ * Version:     $Id: eap.c,v 1.7 2002/04/26 21:35:50 raghu Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@
  *
  */
 
-#include "eap.h"
+#include "rlm_eap.h"
 
 static const char *eap_types[] = {
   "",   
@@ -76,7 +76,7 @@ static const char *eap_types[] = {
  * Load all the required eap authentication types.
  * Get all the supported EAP-types from config file. 
  */
-void eaptype_load(EAP_TYPES **type_list, const char *type_name, CONF_SECTION *cs)
+int eaptype_load(EAP_TYPES **type_list, const char *type_name, CONF_SECTION *cs)
 {
 	EAP_TYPES **last, *node;
 	lt_dlhandle handle;
@@ -87,23 +87,26 @@ void eaptype_load(EAP_TYPES **type_list, const char *type_name, CONF_SECTION *cs
 	snprintf(auth_type_name, sizeof(auth_type_name), "rlm_eap_%s", type_name);
 
 	last = type_list;
+	/* Go to the end of the EAP-Type list, if it is not already loaded */
 	for (node = *type_list; node != NULL; node = node->next) {
 		if (strcmp(node->typename, auth_type_name) == 0)
-			return;
+			return 0;
 		last = &node->next;
 	}
 
+	/* Link the loaded EAP-Type */
 	handle = lt_dlopenext(auth_type_name);
 	if (handle == NULL) {
-		radlog(L_ERR, "rlm_eap: Failed to link to type %s: %s\n", type_name, lt_dlerror());
-		return;
+		radlog(L_ERR, "rlm_eap: Failed to link EAP-Type/%s: %s", 
+				type_name, lt_dlerror());
+		return -1;
 	}
 
-	/* make room for auth type */
+	/* Make room for the EAP-Type */
 	node = (EAP_TYPES *)malloc(sizeof(EAP_TYPES));
 	if (node == NULL) {
 		radlog(L_ERR, "rlm_eap: out of memory");
-		return;
+		return -1;
 	}
 
 	/* fill in the structure */
@@ -124,7 +127,7 @@ void eaptype_load(EAP_TYPES **type_list, const char *type_name, CONF_SECTION *cs
 	if (node->typeid == 0) {
 		radlog(L_ERR, "rlm_eap: Invalid type name %s cannot be linked", type_name);
 		free(node);
-		return;
+		return -1;
 	}
 	
 	node->type = (EAP_TYPE *)lt_dlsym(node->handle, auth_type_name);
@@ -133,7 +136,7 @@ void eaptype_load(EAP_TYPES **type_list, const char *type_name, CONF_SECTION *cs
 				auth_type_name, type_name, lt_dlerror());
 		lt_dlclose(node->handle);	/* ignore any errors */
 		free(node);
-		return;
+		return -1;
 	}
 	if ((node->type->attach) && 
 		((node->type->attach)(node->cs, &(node->type_stuff)) < 0)) {
@@ -141,12 +144,12 @@ void eaptype_load(EAP_TYPES **type_list, const char *type_name, CONF_SECTION *cs
 		radlog(L_ERR, "rlm_eap: Failed to initialize the type %s", type_name);
 		lt_dlclose(node->handle);
 		free(node);
-		return;
+		return -1;
 	}
 
 	radlog(L_INFO, "rlm_eap: Loaded and initialized the type %s", type_name);
 	*last = node;
-	return;
+	return 0;
 }
 
 /*
@@ -212,7 +215,8 @@ int eaptype_call(int eap_type, operation_t action,
 
 /*
  * Based on TYPE, call the appropriate EAP-type handler
- * currently it defaults to MD5-type
+ * Default to the configured EAP-Type 
+ * for all Unsupported EAP-Types 
  */
 int eaptype_select(EAP_TYPES *type_list, EAP_HANDLER *handler, char *conftype)
 {
@@ -231,20 +235,20 @@ int eaptype_select(EAP_TYPES *type_list, EAP_HANDLER *handler, char *conftype)
 	}
 
 	eaptype = &handler->eap_ds->response->type;
-        switch(eaptype->type) {
-        case PW_EAP_IDENTITY:
+	switch(eaptype->type) {
+	case PW_EAP_IDENTITY:
 		if (eaptype_call(type, INITIATE, type_list, handler) == 0)
 			return EAP_INVALID;
-                break;
+			break;
 
-        case PW_EAP_NAK:
-                /*
+	case PW_EAP_NAK:
+		/*
 		 * It is invalid to request identity, notification & nak in nak
 		 */
-                if ((eaptype->data != NULL) &&
-                        (eaptype->data[0] < PW_EAP_MD5)) {
-                        return EAP_INVALID;
-                }
+		if ((eaptype->data != NULL) &&
+			(eaptype->data[0] < PW_EAP_MD5)) {
+			return EAP_INVALID;
+		}
 		switch (eaptype->data[0]) {
 		case PW_EAP_MD5:
 		case PW_EAP_TLS:
@@ -266,19 +270,19 @@ int eaptype_select(EAP_TYPES *type_list, EAP_HANDLER *handler, char *conftype)
 			break;
 		}
 		break;
-        case PW_EAP_MD5:
-        case PW_EAP_OTP:
-        case PW_EAP_GTC:
-        case PW_EAP_TLS:
-        default:
-		radlog(L_INFO, "rlm_eap: EAP_TYPE - %s",
-			eap_types[eaptype->type]);
-                if (eaptype_call(eaptype->type, AUTHENTICATE,
-			type_list, handler) == 0) {
-                        return EAP_INVALID;
-                }
+		case PW_EAP_MD5:
+		case PW_EAP_OTP:
+		case PW_EAP_GTC:
+		case PW_EAP_TLS:
+		default:
+			radlog(L_INFO, "rlm_eap: EAP_TYPE - %s",
+				eap_types[eaptype->type]);
+			if (eaptype_call(eaptype->type, AUTHENTICATE,
+				type_list, handler) == 0) {
+				return EAP_INVALID;
+		}
 		break;
-        }
+	}
 	return EAP_OK;
 }
 
@@ -294,21 +298,22 @@ eap_packet_t *eap_attribute(VALUE_PAIR *vps)
 	VALUE_PAIR *vp_list, *vp;
 	eap_packet_t *eap_packet;
 	unsigned char *ptr;
-	int len;
+	uint16_t len;
 	int total_len;
 
-        vp_list = paircopy2(vps, PW_EAP_MESSAGE);
-        if (vp_list == NULL) {
+	/* Get only EAP-Message attribute list */
+	vp_list = paircopy2(vps, PW_EAP_MESSAGE);
+	if (vp_list == NULL) {
 		radlog(L_ERR, "rlm_eap: EAP_Message not found");
-                return NULL;
+		return NULL;
 	}
 
 	/*
 	 * Get the Actual length from the EAP packet
 	 * First EAP-Message contains the EAP packet header
 	 */
-        memcpy(&len, vp_list->strvalue+2, sizeof(u_short));
-        len = ntohs(len);
+	memcpy(&len, vp_list->strvalue+2, sizeof(uint16_t));
+	len = ntohs(len);
 
 	eap_packet = (eap_packet_t *)malloc(len);
 	if (eap_packet == NULL) {
@@ -328,7 +333,7 @@ eap_packet_t *eap_attribute(VALUE_PAIR *vps)
 		return eap_packet;
 	 */
 
-        if (vp_list->next == NULL) {
+	if (vp_list->next == NULL) {
 		pairfree(&vp_list);
 		return eap_packet;
 	}
@@ -357,7 +362,7 @@ eap_packet_t *eap_attribute(VALUE_PAIR *vps)
 int eap_wireformat(EAP_PACKET *reply)
 {
 	eap_packet_t	*hdr;
-	int		total_length = 0;
+	uint16_t total_length = 0;
 
 	if (reply == NULL) return EAP_INVALID;
 
@@ -379,7 +384,7 @@ int eap_wireformat(EAP_PACKET *reply)
 	hdr->code = (reply->code & 0xFF);
 	hdr->id = (reply->id & 0xFF);
 	total_length = htons(total_length);
-	memcpy(hdr->length, &total_length, sizeof(u_short));
+	memcpy(hdr->length, &total_length, sizeof(uint16_t));
 
 	if (reply->code < 3) {
 		hdr->data[0] = (reply->type.type & 0xFF);
@@ -394,7 +399,7 @@ int eap_wireformat(EAP_PACKET *reply)
 			free(reply->type.data);
 			reply->type.data = reply->packet + EAP_HEADER_LEN + 1/*EAPtype*/;
 		}
-	} 
+	}
 
 	return EAP_VALID;
 }
@@ -404,28 +409,28 @@ int eap_wireformat(EAP_PACKET *reply)
  * If EAP exceeds 253, frame it in multiple EAP-Message attrs.
  * Set the RADIUS reply codes based on EAP request codes
  * Append any additonal VPs to RADIUS reply
- * 
  */
 int eap_compose(REQUEST *request, EAP_PACKET *reply)
 {
-	int eap_len, len;
+	uint16_t eap_len, len;
 	VALUE_PAIR *eap_msg;
 	VALUE_PAIR *vp;
 	eap_packet_t *eap_packet;
-        unsigned char 	*ptr;
+	unsigned char 	*ptr;
 
 	/*
-	 * Either use unique id or we can use the below to get id
-	 * good way is to use uniqe id, and not depend on any thing else 
+	 * Id is expected to be set by the EAP-Type, if not,
+	 * currently RADIUS request Id is used.
+	 * Ideally, a unique Id should be generated.
 	 */
 	if (reply->id == 0) reply->id = request->packet->id;
 
 	if (eap_wireformat(reply) == EAP_INVALID) {
-		return EAP_VALID;
+		return EAP_INVALID;
 	}
 	eap_packet = (eap_packet_t *)reply->packet;
 
-	memcpy(&eap_len, &(eap_packet->length), sizeof(u_short));
+	memcpy(&eap_len, &(eap_packet->length), sizeof(uint16_t));
 	len = eap_len = ntohs(eap_len);
 	ptr = (unsigned char *)eap_packet;
 
@@ -476,21 +481,21 @@ int eap_compose(REQUEST *request, EAP_PACKET *reply)
 		
 	/* Set request reply code */
 	switch(reply->code) {
-		case PW_EAP_SUCCESS:
-			request->reply->code = PW_AUTHENTICATION_ACK;
-			break;
-		case PW_EAP_FAILURE:
-			request->reply->code = PW_AUTHENTICATION_REJECT;
-			break;
-		case PW_EAP_REQUEST:
-			request->reply->code = PW_ACCESS_CHALLENGE;
-			break;
-		case PW_EAP_RESPONSE:
-		default:
-			/* Should never enter here */
-			radlog(L_ERR, "rlm_eap: reply code is unknown, Reject it.");
-			request->reply->code = PW_AUTHENTICATION_REJECT;
-			break;
+	case PW_EAP_SUCCESS:
+		request->reply->code = PW_AUTHENTICATION_ACK;
+		break;
+	case PW_EAP_FAILURE:
+		request->reply->code = PW_AUTHENTICATION_REJECT;
+		break;
+	case PW_EAP_REQUEST:
+		request->reply->code = PW_ACCESS_CHALLENGE;
+		break;
+	case PW_EAP_RESPONSE:
+	default:
+		/* Should never enter here */
+		radlog(L_ERR, "rlm_eap: reply code is unknown, Reject it.");
+		request->reply->code = PW_AUTHENTICATION_REJECT;
+		break;
 	}
 	return 0;
 }
@@ -504,13 +509,13 @@ int eap_start(REQUEST *request)
 	VALUE_PAIR *eap_msg;
 	EAP_DS *eapstart;
 
-        eap_msg = pairfind(request->packet->vps, PW_EAP_MESSAGE);
-        if (eap_msg == NULL) {
+	eap_msg = pairfind(request->packet->vps, PW_EAP_MESSAGE);
+	if (eap_msg == NULL) {
 		radlog(L_ERR, "rlm_eap: EAP-Message not found");
 		return EAP_NOOP;
 	}
 
-        if (eap_msg->length != EAP_START) {
+	if (eap_msg->length != EAP_START) {
 		return EAP_NOTFOUND;
 	}
 
@@ -550,28 +555,28 @@ void eap_success(REQUEST *request, EAP_PACKET *reply)
  */
 int eap_validation(eap_packet_t *eap_packet)
 {
-	int len;
+	uint16_t len;
 
-        memcpy(&len, eap_packet->length, sizeof(u_short));
-        len = ntohs(len);
+	memcpy(&len, eap_packet->length, sizeof(uint16_t));
+	len = ntohs(len);
 
 	/* High level EAP packet checks */
-        if ((len <= EAP_HEADER_LEN) ||
+	if ((len <= EAP_HEADER_LEN) ||
 		(eap_packet->code != PW_EAP_RESPONSE) ||
 		(eap_packet->data[0] <= 0) ||
 		(eap_packet->data[0] > PW_EAP_MAX_TYPES)) {
 
 		radlog(L_AUTH, "rlm_eap: Incorrect EAP Message, "
 				"Ignoring the packet");
-                return EAP_INVALID;
-        }
+		return EAP_INVALID;
+	}
 
 	/* we don't expect notification, but we send it */
 	if (eap_packet->data[0] == PW_EAP_NOTIFICATION) {
 
 		radlog(L_AUTH, "rlm_eap: Got NOTIFICATION, "
 				"Ignoring the packet");
-                return EAP_INVALID;
+		return EAP_INVALID;
 	}
 
 	return EAP_VALID;
@@ -606,7 +611,8 @@ VALUE_PAIR *eap_useridentity(EAP_HANDLER *list, eap_packet_t *eap_packet, unsign
  */
 char *eap_identity(eap_packet_t *eap_packet)
 {
-	int len, size;
+	int size;
+	uint16_t len;
 	char *identity;
 
 	if ((eap_packet == NULL) ||
@@ -615,7 +621,7 @@ char *eap_identity(eap_packet_t *eap_packet)
 		return NULL;
 	}
 
-	memcpy(&len, eap_packet->length, sizeof(u_short));
+	memcpy(&len, eap_packet->length, sizeof(uint16_t));
 	len = ntohs(len);
 
 	if ((len <= 5) || (eap_packet->data[1] == 0x00)) {
@@ -665,13 +671,14 @@ EAP_HANDLER *eap_handler(EAP_HANDLER **list, eap_packet_t **eap_packet_p, REQUES
 		}
 
 		handler = eaplist_isreply(list, unique);
+		free(unique);
+		unique = NULL;
 		if (handler == NULL) {
 			/* Either send EAP_Identity or EAP-Fail */
 			radlog(L_ERR, "rlm_eap: Either EAP-request timed out OR"
 				" EAP-response to an unknown EAP-request");
 			return NULL;
 		}
-		free(unique);
 	} else {
 
 		handler = eap_handler_alloc();
@@ -687,13 +694,54 @@ EAP_HANDLER *eap_handler(EAP_HANDLER **list, eap_packet_t **eap_packet_p, REQUES
 		handler->opaque = NULL;
 		handler->free_opaque = NULL;
 		handler->next = NULL;
-		handler->username = paircopy(request->username);
 
 		handler->identity = eap_identity(eap_packet);
 		if (handler->identity == NULL) {
+			radlog(L_ERR, "rlm_eap: Identity Unknown, authentication failed");
 			eap_handler_free(&handler);
 			return NULL;
 		}
+
+		/* Get the User-Name */
+		if (request->username == NULL) {
+			handler->username = pairmake("User-Name", handler->identity, T_OP_EQ);
+		} else {
+			handler->username = paircopy(request->username);
+		}
+
+		/* No User-Name, No authentication */
+		/*
+		if (handler->username == NULL) {
+			radlog(L_ERR, "rlm_eap: Unknown User, authentication failed");
+			eap_handler_free(&handler);
+			return NULL;
+		}
+		*/
+
+		/*
+		 * Always get the configured values, for each user.
+		 * to pass it to the specific EAP-Type
+		 *
+		 * No Configured information found for a user, means
+		 * there is no such user in the database.
+		 *
+		 * Every user should have, atleast, one item configured
+		 * This is required for Authentication purpose.
+		 */
+		handler->configured = paircopy(request->config_items);
+		if (handler->configured == NULL) {
+			radlog(L_INFO, "rlm_eap: No configured information for this user");
+
+			/*
+			 * FIXME: If there is no config info then
+			 * config_items should provide the same username
+			 * if the user is present in the database.
+			 */
+			/*
+			eap_handler_free(&handler);
+			return NULL;
+			*/
+		}
 	}
 
 	handler->eap_ds = eap_buildds(eap_packet_p);
@@ -713,7 +761,8 @@ EAP_DS *eap_buildds(eap_packet_t **eap_packet_p)
 {
 	EAP_DS *eap_ds = NULL;
 	eap_packet_t	*eap_packet = NULL;
-	int len, typelen;
+	int typelen;
+	uint16_t len;
 
 	eap_packet = *eap_packet_p;
 	if (eap_packet == NULL) {
@@ -728,9 +777,9 @@ EAP_DS *eap_buildds(eap_packet_t **eap_packet_p)
         eap_ds->response->id = eap_packet->id;
         eap_ds->response->type.type = eap_packet->data[0];
 
-	memcpy(&len, eap_packet->length, sizeof(u_short));
+	memcpy(&len, eap_packet->length, sizeof(uint16_t));
 	len = ntohs(len);
-        eap_ds->response->length = len;
+	eap_ds->response->length = len;
 
 	/* First byte in eap_packet->data is *EAP-Type* */
 	/*
@@ -738,7 +787,7 @@ EAP_DS *eap_buildds(eap_packet_t **eap_packet_p)
 	 * The rest is TypeData
 	 * skip *type* while getting typedata from data
 	 */
-        typelen = len - 5/*code+id+length+type*/;
+	typelen = len - 5/*code+id+length+type*/;
 	if (typelen > 0) {
 		/*
 		 * Since packet contians the complete eap_packet, 
