@@ -1,7 +1,7 @@
 /*
  * eap.h    Header file containing the interfaces for all EAP types.
  *
- * Version:     $Id: eap.h,v 1.11 2002/06/14 14:54:29 aland Exp $
+ * Version:     $Id: eap.h,v 1.13 2002/07/08 12:29:17 pam Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -35,8 +35,8 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "conffile.h"
 #include "radiusd.h"
+#include "modules.h"
 
 
 #define PW_EAP_REQUEST		1
