@@ -1,7 +1,7 @@
 /*
  * eap_md5.c  EAP MD5 functionality.
  *
- * Version:     $Id: eap_md5.c,v 1.6 2002/04/26 21:44:52 raghu Exp $
+ * Version:     $Id: eap_md5.c,v 1.7 2002/07/08 12:05:08 pam Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
  */
 
 #include <stdio.h>
-#include <malloc.h>
+#include <stdlib.h>
 #include "eap.h"
 
 #include "eap_md5.h"
