@@ -1,7 +1,7 @@
 /*
  * rlm_mschap.c	
  *
- * Version:	$Id: rlm_mschap.c,v 1.22 2002/01/21 11:20:42 3APA3A Exp $
+ * Version:	$Id: rlm_mschap.c,v 1.27 2002/06/21 19:57:27 cparker Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -56,6 +56,7 @@
 #include	"modules.h"
 
 #include	"des.h"
+#include        "md4.h"
 #include        "md5.h"
 #include	"sha1.h"
 #include	"smbpass.h"
@@ -104,10 +105,10 @@ static void mppe_GetMasterKey(uint8_t *nt_hashhash,uint8_t *nt_response,
 static void mppe_GetAsymmetricStartKey(uint8_t *masterkey,uint8_t *sesskey,
                                int keylen,int issend);
 
+#if 0
 static void mppe_gen_respkey(uint8_t* secret,uint8_t* vector,
                        uint8_t* salt,uint8_t* enckey,uint8_t* key);
-
-void md4_calc (unsigned char *, unsigned char *, unsigned int);
+#endif
 
 
 
@@ -187,7 +188,7 @@ static void lmpwdhash (char *szHash, const char *szPassword)
 
 	memset(szOEMPass, 0, 14);
 	for (i = 0; i < 14 && szPassword[i]; i++)
-		szOEMPass[i] = toupper(szPassword[i]);
+		szOEMPass[i] = toupper((int) szPassword[i]);
 
 	/* Obtain DES hash of OEM password */
 	des_encrypt(stdText, szOEMPass, szHash); 
@@ -434,13 +435,13 @@ static void mppe_chap2_gen_keys128(uint8_t *secret,uint8_t *vector,
 {
 	uint8_t enckey1[16];
 	uint8_t enckey2[16];
-	uint8_t salt[2];
+	/* uint8_t salt[2]; */
 	uint8_t nt_hashhash[16];
 
 	md4_calc(nt_hashhash,nt_hash,16);
 
 	mppe_chap2_get_keys128(nt_hashhash,response,enckey1,enckey2);
-
+#if 0
 	salt[0] = (vector[0] ^ vector[1] ^ 0x3A) | 0x80;
 	salt[1] = (vector[2] ^ vector[3] ^ vector[4]);
 
@@ -450,6 +451,9 @@ static void mppe_chap2_gen_keys128(uint8_t *secret,uint8_t *vector,
 	salt[1] = (vector[5] ^ vector[6] ^ vector[7]);
 
 	mppe_gen_respkey(secret,vector,salt,enckey2,recvkey);
+#endif
+	memcpy (sendkey, enckey1, 16);
+	memcpy (recvkey, enckey2, 16);
 }
 
 static void mppe_chap2_get_keys128(uint8_t *nt_hashhash,uint8_t *nt_response,
@@ -544,6 +548,10 @@ static void mppe_GetAsymmetricStartKey(uint8_t *masterkey,uint8_t *sesskey,
        memcpy(sesskey,digest,keylen);
 }
 
+
+#if 0
+/*	Not requiered, because encoding will be performed by
+	tunnel_pwencode */
 static void mppe_gen_respkey(uint8_t* secret,uint8_t* vector,
                        uint8_t* salt,uint8_t* enckey,uint8_t* key)
 {
@@ -581,6 +589,7 @@ static void mppe_gen_respkey(uint8_t* secret,uint8_t* vector,
        memcpy(key,salt,2);
        memcpy(key + 2,plain,32);
 }
+#endif /* 0 */
 
 
 /*
@@ -853,10 +862,18 @@ static int mschap_authenticate(void * instance, REQUEST *request)
 						smbPasswd.smb_nt_passwd,
 						response->strvalue + 26,
 						mppe_sendkey,mppe_recvkey);
+
+					mppe_add_reply( &request->reply->vps,
+						"MS-MPPE-Recv-Key",mppe_recvkey,16);
+					mppe_add_reply( &request->reply->vps,
+						"MS-MPPE-Send-Key",mppe_sendkey,16);
+
+/*
 					mppe_add_reply( &request->reply->vps,
 						"MS-MPPE-Recv-Key",mppe_recvkey,34);
 					mppe_add_reply( &request->reply->vps,
 						"MS-MPPE-Send-Key",mppe_sendkey,34);
+*/
 				}
 				reply_attr = pairmake("MS-MPPE-Encryption-Policy",
 					(inst->require_encryption)? "0x00000002":"0x00000001",
