@@ -1,7 +1,7 @@
 /*
  * rlm_pap.c
  *
- * Version:  $Id: rlm_pap.c,v 1.5 2002/06/25 17:32:41 kkalev Exp $
+ * Version:  $Id: rlm_pap.c,v 1.7 2002/07/11 19:35:46 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@
 	free((char *)inst->scheme); \
 	free(inst)
 
-static const char rcsid[] = "$Id: rlm_pap.c,v 1.5 2002/06/25 17:32:41 kkalev Exp $";
+static const char rcsid[] = "$Id: rlm_pap.c,v 1.7 2002/07/11 19:35:46 aland Exp $";
 
 /*
  *      Define a structure for our module configuration.
@@ -58,7 +58,9 @@ static const char rcsid[] = "$Id: rlm_pap.c,v 1.5 2002/06/25 17:32:41 kkalev Exp
 typedef struct rlm_pap_t {
         char *scheme;  /* password encryption scheme */
 	int sch;
+#if HAVE_PTHREAD_H
 	pthread_mutex_t mutex;	
+#endif
 } rlm_pap_t;
 
 /*
@@ -122,7 +124,9 @@ static int pap_instantiate(CONF_SECTION *conf, void **instance)
 		inst->sch = PAP_ENC_CLEAR;
 	else if (strcasecmp(inst->scheme,"crypt") == 0){
 		inst->sch = PAP_ENC_CRYPT;
+#if HAVE_PTHREAD_H
 		pthread_mutex_init(&inst->mutex, NULL);
+#endif
 	}
 	else if (strcasecmp(inst->scheme,"md5") == 0)
 		inst->sch = PAP_ENC_MD5;
@@ -153,7 +157,7 @@ static int pap_authenticate(void *instance, REQUEST *request)
 	MD5_CTX md5_context;
 	SHA1_CTX sha1_context;
 	char digest[20];
-	char buff[21];
+	char buff[MAX_STRING_LEN];
 	rlm_pap_t *inst = (rlm_pap_t *) instance;
 
 	/* quiet the compiler */
@@ -216,19 +220,25 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			break;
 		case PAP_ENC_CRYPT:
 			DEBUG("rlm_pap: Using CRYPT encryption.");
+#if HAVE_PTHREAD_H
 			pthread_mutex_lock(&inst->mutex);
+#endif
 			if (strncmp((char *) passwd_item->strvalue,
 				crypt((char *) request->password->strvalue, (char *)passwd_item->strvalue),
 					passwd_item->length) != 0){
+#if HAVE_PTHREAD_H
 				pthread_mutex_unlock(&inst->mutex);
+#endif
 				DEBUG("rlm_pap: Passwords don't match");
 				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: CRYPT password check failed");
 				module_fmsg_vp = pairmake("Module-Failure-Message",module_fmsg, T_OP_EQ);
 				pairadd(&request->packet->vps, module_fmsg_vp);
 				return RLM_MODULE_REJECT;
 			}
+#if HAVE_PTHREAD_H
 			else
 				pthread_mutex_unlock(&inst->mutex);
+#endif
 			break;
 		case PAP_ENC_MD5:
 
@@ -237,7 +247,7 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			MD5Update(&md5_context, request->password->strvalue, request->password->length);
 			MD5Final(digest, &md5_context);
 			pap_hexify(buff,digest,16);
-			buff[16] = '\0';
+			buff[32] = '\0';
 			if (strncmp((char *)passwd_item->strvalue, buff, passwd_item->length) != 0){
 				DEBUG("rlm_pap: Passwords don't match");
 				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: MD5 password check failed");
@@ -253,7 +263,7 @@ static int pap_authenticate(void *instance, REQUEST *request)
 			SHA1Update(&sha1_context, request->password->strvalue, request->password->length);
 			SHA1Final(digest,&sha1_context);
 			pap_hexify(buff,digest,20);
-			buff[21] = '\0';
+			buff[40] = '\0';
 			if (strncmp((char *)passwd_item->strvalue, buff, passwd_item->length) != 0){
 				DEBUG("rlm_pap: Passwords don't match");
 				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: SHA1 password check failed");
@@ -273,7 +283,9 @@ static int pap_detach(void *instance)
 {
 	rlm_pap_t *inst = (rlm_pap_t *) instance;
 
+#if HAVE_PTHREAD_H
 	pthread_mutex_destroy(&inst->mutex);
+#endif
 	PAP_INST_FREE(inst);
 	return 0;
 }
