@@ -1,7 +1,7 @@
 /*
  * rlm_realm.c	
  *
- * Version:	$Id: rlm_realm.c,v 1.39 2002/05/29 16:18:36 aland Exp $
+ * Version:	$Id: rlm_realm.c,v 1.41 2002/07/18 17:09:18 cparker Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@
 #include "radiusd.h"
 #include "modules.h"
 
-static const char rcsid[] = "$Id: rlm_realm.c,v 1.39 2002/05/29 16:18:36 aland Exp $";
+static const char rcsid[] = "$Id: rlm_realm.c,v 1.41 2002/07/18 17:09:18 cparker Exp $";
 
 #define  REALM_FORMAT_PREFIX   0
 #define  REALM_FORMAT_SUFFIX   1
@@ -184,6 +184,13 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 	DEBUG2("  rlm_realm: Proxying request from user %s to realm %s",
 	       username, realm->realm);
 
+	/*
+	 *	Add the realm name to the request.
+	 */
+	pairadd(&request->packet->vps, pairmake("Realm", realm->realm,
+						T_OP_EQ));
+	DEBUG2("    rlm_realm: Adding Realm = \"%s\"", realm->realm);
+
 	/*
 	 *	Figure out what to do with the request.
 	 */
@@ -225,13 +232,6 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 		break;
 	}
 
-	/*
-	 *	Add the realm name to the request.
-	 */
-	pairadd(&request->packet->vps, pairmake("Realm", realm->realm,
-						T_OP_EQ));
-	DEBUG2("    rlm_realm: Adding Realm = \"%s\"", realm->realm);
-
 	/*
 	 *	Local realm, don't proxy it.
 	 */
@@ -239,6 +239,26 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 		return NULL;
 	}
 
+	/*
+	 *      If this request has arrived from another freeradius server
+	 *      that has already proxied the request, we don't need to do
+	 *      it again.
+	 */
+	for (vp = request->packet->vps; vp; vp = vp->next) {
+		if (vp->attribute == PW_FREERADIUS_PROXIED_TO) {
+			if (request->packet->code == PW_AUTHENTICATION_REQUEST &&
+			    vp->lvalue == realm->ipaddr) {
+				DEBUG2("rlm_realm: Request not proxied due to Freeradius-Proxied-To");
+				return NULL;
+			}
+			if (request->packet->code == PW_ACCOUNTING_REQUEST &&
+			    vp->lvalue == realm->acct_ipaddr) {
+				DEBUG2("rlm_realm: Request not proxied due to Freeradius-Proxied-To");
+				return NULL;
+			}
+		}
+        }
+
 	return realm;
 }
 
