@@ -2,7 +2,7 @@
  * rlm_sql.c		SQL Module
  * 		Main SQL module file. Most ICRADIUS code is located in sql.c
  *
- * Version:	$Id: rlm_sql.c,v 1.87 2002/06/10 10:32:17 fcusack Exp $
+ * Version:	$Id: rlm_sql.c,v 1.92 2002/07/25 15:54:23 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-	"$Id: rlm_sql.c,v 1.87 2002/06/10 10:32:17 fcusack Exp $";
+	"$Id: rlm_sql.c,v 1.92 2002/07/25 15:54:23 aland Exp $";
 
 #include "autoconf.h"
 
@@ -128,30 +128,45 @@ static int sql_xlat(void *instance, REQUEST *request, char *fmt, char *out, int
 	sqlsocket = sql_get_socket(inst);
 	if (sqlsocket == NULL)
 		return 0;
-	if ((inst->module->sql_select_query)(sqlsocket,inst->config,querystr) <0){
+	if (rlm_sql_select_query(sqlsocket,inst,querystr)){
 		radlog(L_ERR, "rlm_sql: database query error");
 		sql_release_socket(inst,sqlsocket);
 		return 0;
 	}
-	row = (inst->module->sql_fetch_row)(sqlsocket, inst->config);
+
+	ret = rlm_sql_fetch_row(sqlsocket, inst);
 	(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
-	sql_release_socket(inst,sqlsocket);
-	if (row == NULL){
-		DEBUG("rlm_sql: SQL query did not return any results");
+
+	if (ret) {
+		DEBUG("rlm_sql: SQL query did not succeed");
+		sql_release_socket(inst,sqlsocket);
 		return 0;
 	}
+
+	row = sqlsocket->row;
+	if (row == NULL) {
+		DEBUG("rlm_sql: SQL query did not return any results");
+		sql_release_socket(inst,sqlsocket);
+		return 0;
+	}
+
 	if (row[0] == NULL){
 		DEBUG("rlm_sql: row[0] returned NULL");
+		sql_release_socket(inst,sqlsocket);
 		return 0;
 	}
 	ret = strlen(row[0]);
 	if (ret > freespace){
 		DEBUG("rlm_sql: sql_xlat:: Insufficient string space");
+		sql_release_socket(inst,sqlsocket);
 		return 0;
 	}
+
 	strncpy(out,row[0],ret);
+
 	DEBUG("rlm_sql: - sql_xlat finished");
 
+	sql_release_socket(inst,sqlsocket);
 	return ret;
 }
 
@@ -175,7 +190,7 @@ static int sql_escape_func(char *out, int outlen, const char *in)
 		 *	mime-encoded equivalents.
 		 */
 		if ((in[0] < 32) ||
-		    strchr("@abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-_: ", *in) == NULL) {
+		    strchr("@abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-_: =/", *in) == NULL) {
 			snprintf(out, outlen, "=%02X", (unsigned char) in[0]);
 			in++;
 			out += 3;
@@ -325,6 +340,7 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 	SQL_INST *inst = instance;
 	SQL_ROW row;
 	char    querystr[MAX_QUERY_LEN];
+	int	ret;
 
 	/* sqlusername holds the sql escaped username. The original
 	 * username is at most MAX_STRING_LEN chars long and
@@ -418,21 +434,19 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
 		return RLM_MODULE_NOTFOUND;
 	}
-#if 0 /* FIXME: Debug being printed elsewhere? */
+
 	/*
 	 * Uncomment these lines for debugging
 	 * Recompile, and run 'radiusd -X'
-	 *
-	 DEBUG2("rlm_sql:  check items");
-	 vp_printlist(stderr, check_tmp);
-	 DEBUG2("rlm_sql:  reply items");
-	 vp_printlist(stderr, reply_tmp);
 	 */
-#endif
 
-#if 0 /* FIXME: this is the *real* authorizing */
-	vp_printlist(stderr, check_tmp);
-#endif
+	/* 
+	DEBUG2("rlm_sql:  check items");
+	vp_listdebug(check_tmp);
+	DEBUG2("rlm_sql:  reply items");
+	vp_listdebug(reply_tmp); 
+	*/
+
 	if (paircmp(request, request->packet->vps, check_tmp, &reply_tmp) != 0) {
 		radlog(L_INFO, "rlm_sql: Pairs do not match [%s]", sqlusername);
 		/* Remove the username we (maybe) added above */
@@ -454,16 +468,22 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
 
-		if ((inst->module->sql_select_query)(sqlsocket, inst->config, querystr) < 0) {
+		if (rlm_sql_select_query(sqlsocket, inst, querystr)) {
 			radlog(L_ERR, "rlm_sql_authorize: database query error");
 			sql_release_socket(inst, sqlsocket);
 			return RLM_MODULE_FAIL;
 		}
+		ret = rlm_sql_fetch_row(sqlsocket, inst);
 
-		row = (inst->module->sql_fetch_row)(sqlsocket, inst->config);
 		(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 		sql_release_socket(inst, sqlsocket);
 
+		if (ret) {
+			radlog(L_ERR, "rlm_sql_authorize: query failed");
+			return RLM_MODULE_FAIL;
+		}
+
+		row = sqlsocket->row;
 		if (row == NULL) {
 			radlog(L_ERR, "rlm_sql_authorize: no rows returned from query (no such user)");
 			return RLM_MODULE_OK;
@@ -531,7 +551,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			if (sqlsocket == NULL)
 				return(RLM_MODULE_FAIL);
 			if (querystr) {
-				if ((inst->module->sql_query)(sqlsocket, inst->config, querystr) < 0)
+				if (rlm_sql_query(sqlsocket, inst, querystr))
 					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting for Acct On/Off packet - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
 				(inst->module->sql_finish_query)(sqlsocket, inst->config);
 			}
@@ -555,7 +575,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			if (sqlsocket == NULL)
 				return(RLM_MODULE_FAIL);
 			if (querystr) {
-				if ((inst->module->sql_query)(sqlsocket, inst->config, querystr) < 0)
+				if (rlm_sql_query(sqlsocket, inst, querystr))
 					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting for ALIVE packet - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
 				(inst->module->sql_finish_query)(sqlsocket, inst->config);
 			}
@@ -579,7 +599,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			if (sqlsocket == NULL)
 				return(RLM_MODULE_FAIL);
 			if (querystr) {
-				if ((inst->module->sql_query)(sqlsocket, inst->config, querystr) < 0) {
+				if (rlm_sql_query(sqlsocket, inst, querystr)) {
 					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting" " for START packet - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
 
 					/*
@@ -591,7 +611,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 					query_log(inst, querystr);
 
 					if (querystr) {
-						if ((inst->module->sql_query)(sqlsocket, inst->config, querystr) < 0) {
+						if (rlm_sql_query(sqlsocket, inst, querystr)) {
 							radlog(L_ERR, "rlm_sql: Couldn't update SQL" "accounting START record - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
 						}
 						(inst->module->sql_finish_query)(sqlsocket, inst->config);
@@ -618,7 +638,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			if (sqlsocket == NULL)
 				return(RLM_MODULE_FAIL);
 			if (querystr) {
-				if ((inst->module->sql_query)(sqlsocket, inst->config, querystr) < 0) {
+				if (rlm_sql_query(sqlsocket, inst, querystr)) {
 					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting STOP record - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
 				}
 				else {
@@ -652,7 +672,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 						query_log(inst, querystr);
 
 						if (querystr) {
-							if ((inst->module->sql_query)(sqlsocket, inst->config, querystr) < 0) {
+							if (rlm_sql_query(sqlsocket, inst, querystr)) {
 								radlog(L_ERR, "rlm_sql: Couldn't insert SQL accounting STOP record - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
 							}
 							(inst->module->sql_finish_query)(sqlsocket, inst->config);
@@ -689,6 +709,7 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
         uint32_t        ipno = 0;
         char            *call_num = NULL;
 	VALUE_PAIR      *vp;
+	int		ret;
 
 	/* If simul_count_query is not defined, we don't do any checking */
 	if (inst->config->simul_count_query[0] == 0) {
@@ -709,16 +730,31 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
 		return RLM_MODULE_FAIL;
 
 	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->simul_count_query, request, NULL);
-	if((inst->module->sql_select_query)(sqlsocket, inst->config, querystr) < 0) {
+	if(rlm_sql_select_query(sqlsocket, inst, querystr)) {
 		radlog(L_ERR, "sql_checksimul: Database query failed");
 		sql_release_socket(inst, sqlsocket);
 		return RLM_MODULE_FAIL;
 	}
 
-	row = (inst->module->sql_fetch_row)(sqlsocket, inst->config);
-	request->simul_count = atoi(row[0]);
+	ret = rlm_sql_fetch_row(sqlsocket, inst);
+
+	if (ret == 0) {
+		row = sqlsocket->row;
+	}
 	(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
 
+	if (ret) {
+		sql_release_socket(inst, sqlsocket);
+		return RLM_MODULE_FAIL;
+	}
+
+	if (row == NULL) {
+		sql_release_socket(inst, sqlsocket);
+		return RLM_MODULE_FAIL;
+	}
+
+	request->simul_count = atoi(row[0]);
+
 	if(request->simul_count < request->simul_max) {
 		sql_release_socket(inst, sqlsocket);
 		return RLM_MODULE_OK;
@@ -732,7 +768,7 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
 	}
 
 	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->simul_verify_query, request, NULL);
-	if((inst->module->sql_select_query)(sqlsocket, inst->config, querystr) < 0) {
+	if(rlm_sql_select_query(sqlsocket, inst, querystr)) {
 		radlog(L_ERR, "sql_checksimul: Database query error");
 		sql_release_socket(inst, sqlsocket);
 		return RLM_MODULE_FAIL;
@@ -749,7 +785,10 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
                 call_num = vp->strvalue;        
 
 
-	while((row=(inst->module->sql_fetch_row)(sqlsocket, inst->config))) {
+	while (rlm_sql_fetch_row(sqlsocket, inst) == 0) {
+		row = sqlsocket->row;
+		if (row == NULL)
+			break;
 		check = rad_check_ts((uint32_t)row[3], (int)row[4], row[2], row[1]);
 
                 /*
@@ -782,7 +821,10 @@ static int rlm_sql_checksimul(void *instance, REQUEST * request) {
 								row[1],row[3],row[4]);
 				sqlsocket1 = sql_get_socket(inst);
 				sprintf(querystr, inst->config->simul_zap_query, row[0]);
-				(inst->module->sql_query)(sqlsocket1, inst->config, querystr);
+				if (rlm_sql_query(sqlsocket1, inst, querystr)) {
+					radlog(L_ERR, "rlm_sql: Deletion of stale session [%s] failed",
+								row[1]);
+				}
 				(inst->module->sql_finish_select_query)(sqlsocket1, inst->config);
 				sql_release_socket(inst, sqlsocket1);
 			}
