@@ -1,6 +1,6 @@
 /*
  * x99_pwe.c
- * $Id: x99_pwe.c,v 1.14 2002/06/24 10:13:58 fcusack Exp $
+ * $Id: x99_pwe.c,v 1.17 2002/07/10 08:25:59 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,13 +21,14 @@
 
 /*
  * This file implements password checking functions for each supported
- * encoding (PAP, CHAP, etc.).  It is expected to be temporary until a
- * libradius modular interface can be created.  The current libradius
- * interface is not sufficient for x9.9 use.
+ * encoding (PAP, CHAP, etc.).  The current libradius interface is not
+ * sufficient for x9.9 use.
  */
 
 #ifdef FREERADIUS
-#include "autoconf.h"
+#define _LRAD_MD4_H
+#define _LRAD_MD5_H
+#define _LRAD_SHA1_H
 #include "libradius.h"
 #include "rad_assert.h"
 #endif
@@ -41,7 +42,7 @@
 
 #include <string.h>
 
-static const char rcsid[] = "$Id: x99_pwe.c,v 1.14 2002/06/24 10:13:58 fcusack Exp $";
+static const char rcsid[] = "$Id: x99_pwe.c,v 1.17 2002/07/10 08:25:59 fcusack Exp $";
 
 
 /* Attribute IDs for supported password encodings. */
