@@ -1,6 +1,6 @@
 /*
  * x99_rlm.c
- * $Id: x99_rlm.c,v 1.23 2002/07/03 08:42:04 fcusack Exp $
+ * $Id: x99_rlm.c,v 1.29 2002/07/19 23:27:44 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -32,29 +32,27 @@
  */
 
 /*
- * TODO: support max usage limit (x iterations)
  * TODO: support soft PIN? ???
  * TODO: support other than ILP32 (for State)
  */
 
-#ifdef FREERADIUS
-#include "autoconf.h"
-#include "libradius.h"
-#include "radiusd.h"
-#include "modules.h"
-#include "conffile.h"
-#endif
-#include "x99.h"
 
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
 #include <unistd.h>
+#include <fcntl.h>
+#include <time.h>
 #include <netinet/in.h>	/* htonl() */
 
-static const char rcsid[] = "$Id: x99_rlm.c,v 1.23 2002/07/03 08:42:04 fcusack Exp $";
+#ifdef FREERADIUS
+#include "radiusd.h"
+#include "modules.h"
+#endif
+#include "x99.h"
+
+static const char rcsid[] = "$Id: x99_rlm.c,v 1.29 2002/07/19 23:27:44 fcusack Exp $";
 
 /* Global data */
 static int rnd_fd;			/* fd for random device           */
@@ -166,9 +164,10 @@ x99_token_instantiate(CONF_SECTION *conf, void **instance)
 		CHALLENGE_PROMPT);
     }
 
-    if (data->softfail < 1 ) {
+    if (data->softfail < 0 ) {
 	data->softfail = 5;
-	x99_log(X99_LOG_ERR, "softfail must be at least 1, using default of 5");
+	x99_log(X99_LOG_ERR, "softfail must be at least 1 "
+		"(or 0 == infinite), using default of 5");
     }
 
     if (data->hardfail < 0 ) {
@@ -183,6 +182,13 @@ x99_token_instantiate(CONF_SECTION *conf, void **instance)
 		"fast_sync is yes, but allow_sync is no; disabling fast_sync");
     }
 
+    if (!data->allow_sync && !data->allow_async) {
+	x99_log(X99_LOG_ERR,
+		"at least one of {allow_async, allow_sync} must be set");
+	free(data);
+	return -1;
+    }
+
     if (data->ewindow_size > MAX_EWINDOW_SIZE || data->ewindow_size < 0) {
 	data->ewindow_size = 0;
 	x99_log(X99_LOG_ERR, "max event window size is %d, using default of 0",
@@ -230,10 +236,12 @@ x99_token_instantiate(CONF_SECTION *conf, void **instance)
     if (stat(data->syncdir, &st) != 0) {
 	x99_log(X99_LOG_ERR, "syncdir %s error: %s",
 		data->syncdir, strerror(errno));
+	free(data);
 	return -1;
     }
     if (st.st_mode != (S_IFDIR|S_IRWXU)) {
 	x99_log(X99_LOG_ERR, "syncdir %s has loose permissions", data->syncdir);
+	free(data);
 	return -1;
     }
 
@@ -329,7 +337,7 @@ x99_token_authorize(void *instance, REQUEST *request)
 	     * to set a value, /existence/ of the vp is the signal.
 	     */
 	    if ((vp = paircreate(PW_X99_FAST, PW_TYPE_INTEGER)) == NULL) {
-		x99_log(X99_LOG_ERR, "autz: no memory");
+		x99_log(X99_LOG_CRIT, "autz: no memory");
 		return RLM_MODULE_FAIL;
 	    }
 	    pairadd(&request->config_items, vp);
@@ -620,7 +628,8 @@ sync_response:
 		x99_log(X99_LOG_ERR,
 			"auth: unable to get sync data e:%d t:%d for [%s]",
 			i, 0, username);
-		return RLM_MODULE_FAIL;
+		rc = RLM_MODULE_FAIL;
+		goto return_pw_valid;
 		/* NB: last_auth, failcount not updated. */
 	    }
 
@@ -630,7 +639,8 @@ sync_response:
 		x99_log(X99_LOG_ERR, "auth: unable to calculate sync response "
 			"e:%d t:%d for [%s], to challenge %s",
 			i, 0, username, challenge);
-		return RLM_MODULE_FAIL;
+		rc = RLM_MODULE_FAIL;
+		goto return_pw_valid;
 		/* NB: last_auth, failcount not updated. */
 	    }
 	    DEBUG("rlm_x99_token: auth: [%s], sync challenge %d %s, "
@@ -642,13 +652,13 @@ sync_response:
 		 * Yay!  User authenticated via sync mode.  Resync.
 		 *
 		 * The same failure/replay issue applies here as in the
-		 * identical code block in the async section, above, with
+		 * identical code block in the async section above, with
 		 * the additional problem that a response can be reused
 		 * indefinitely!  (until the sync data is updated)
 		 */
 		rc = RLM_MODULE_OK;
 		if (x99_get_sync_data(inst->syncdir,username,user_info.card_id,
-				      1, 0 ,challenge,user_info.keyblock) != 0){
+				      1, 0, challenge,user_info.keyblock) != 0){
 		    x99_log(X99_LOG_ERR, "auth: unable to get sync data "
 			    "e:%d t:%d for [%s] (for resync)", 1, 0, username);
 		    rc = RLM_MODULE_FAIL;
