@@ -1,6 +1,6 @@
 /*
  * x99_sync.h
- * $Id: x99_sync.h,v 1.2 2002/01/17 04:02:18 fcusack Exp $
+ * $Id: x99_sync.h,v 1.4 2002/07/03 08:42:04 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 #ifndef X99_SYNC_H
 #define X99_SYNC_H
 
+static int x99_get_failcount(const char *syncdir, const char *username,
+			     int *failcount);
 static char * x99_acquire_sd_lock(const char *syncdir, const char *username);
 static void x99_release_sd_lock(char *lockfile);
 
